{% load i18n %}
{% load skillfarm %}

{% skillfarm_static 'js/skillfarm.js' %}
{% skillfarm_static 'js/modals/modal-system.js' %}
{% skillfarm_static 'js/modals/skillfarm-confirm.js' %}
{% skillfarm_static 'js/modals/skillfarm-skillset.js' %}
